/*	
	Proyecto TFG:
		AMH VIDA
	Ao:
		2011-2012.
	Version:
		1.0
	Fecha de ultima modificacion:
		18-Jul-2012.
	Autor:
		Jose Luis Martinez Jimenez.
	Tutores:
		Francisco Javier Finat Codes.
		Valentn Cardeoso Payo
	Descripcion:
		Ventana principal del proyecto.
*/
#include "MainWindow.h"
#include "trackcontroller.h"
#include "renderwidget.h"
#include "settings.h"
#include "processingthread.h"
#include <QTimer>
#include <QToolBar>
#include <QDockWidget>

MainWindow::MainWindow() : QMainWindow(0) {
    ui.setupUi(this);
	renderWidget = new RenderWidget(this);
	trackController = new TrackController();
	trackController->setRootFilter(renderWidget);
	setCentralWidget(renderWidget);

	// create the toolbar
	cameraToolbar = new QToolBar("Camera Toolbar", this);
	cameraToolbar->setIconSize(QSize(32,32));
	cameraToolbar->setToolButtonStyle(Qt::ToolButtonIconOnly);
	cameraToolbar->addAction(ui.actionStart);
	cameraToolbar->addAction(ui.actionRecord);
	cameraToolbar->addAction(ui.actionStop);
	addToolBar(Qt::BottomToolBarArea, cameraToolbar);
	
	// create the settings dock widget
	settingsDock = new QDockWidget("Configuracin", this);
	settingsDock->setAllowedAreas(Qt::RightDockWidgetArea | Qt::LeftDockWidgetArea);
	settingsWidget = new SettingsWidget();
	settingsDock->setWidget(settingsWidget);
	
	settingsWidget->setTrackController(trackController);
	addDockWidget(Qt::RightDockWidgetArea, settingsDock);
	connect(settingsWidget, SIGNAL(resolutionSelected(CaptureThread::FrameSize)), this, SLOT(onResolutionSelected(CaptureThread::FrameSize)));
	connect(settingsWidget, SIGNAL(prevViewSelected(bool)), this, SLOT(onPrevViewSelected(bool)));
	
	//connect(settingsWidget, SIGNAL(maskChanged(bool)), this, SLOT(onMaskChanged(bool)));
	//connect(settingsWidget, SIGNAL(skeletonChanged(bool)), this, SLOT(onSkeletonChanged(bool)));
	
	Ret *r=Ret::getRet();
	connect(r/*trackController->imageBuffer*/, SIGNAL(bufferEmpty()), this, SLOT(stopTracking()));
	
	updateTimer = new QTimer(this);
	//connect(updateTimer, SIGNAL(timeout()), this, SLOT(updateStats()));
	//updateTimer->start(1000);
	
	connect(ui.actionQuit, SIGNAL(triggered()), this, SLOT(close()));
	connect(ui.actionCerrar_video, SIGNAL(triggered()), this, SLOT(closeFile()));
	connect(ui.actionStart, SIGNAL(triggered()), this, SLOT(startTracking()));
	connect(ui.actionStop, SIGNAL(triggered()), this, SLOT(stopTracking()));
	connect(ui.actionRecord, SIGNAL(triggered()), this, SLOT(pauseTracking()));
	connect(ui.actionAbrir, SIGNAL(triggered()), this, SLOT(openFile()));
	connect(ui.actionAcerca_de,SIGNAL(activated()),this,SLOT(acercaDeAction()));
	
	ui.actionCerrar_video->setEnabled(false);
	fileOpened = false;
	ui.actionRecord->setEnabled(false);
}

void MainWindow::acercaDeAction(){
    QMessageBox::StandardButton reply;
    reply = QMessageBox::information(this, tr("Acerca de"), "Trabajo fin de grado\n\nAlumno:\nJos Luis Martnez Jimnez\n\nTutores:\nFranciscoJavier Finat Codes\nValentin Cardeoso Payo"); 
}

void MainWindow::closeFile() {
	stopTracking();
	if(fileOpened == true){
		fileOpened = false;
		//trackController->closeFile();
	}
	ui.actionCerrar_video->setEnabled(false);
}

void MainWindow::closeEvent(QCloseEvent*) {
	if(trackController->isTracking()) {
		trackController->stopTracking();
	}
}

//// flip the image vertically
//void MainWindow::onFlipVerticalChanged(bool flip) {
//	trackController->getProcessingThread()->setFlipVertical(flip);
//}

void MainWindow::onPrevViewSelected(bool selected){
	if(!trackController->isTracking()){
		trackController->setPrevView(selected);
	}
}

// resolution has been changed from the settings
void MainWindow::onResolutionSelected(CaptureThread::FrameSize newSize) {
	if(trackController->isTracking()) {
		trackController->stopTracking();
		trackController->setFrameSize(newSize);
		trackController->startTracking();
	}
	else {
		trackController->setFrameSize(newSize);
	}
}

void MainWindow::pauseTracking() {
	Ret *r=Ret::getRet();
	r->setPause(true);
	trackController->pauseTracking();
	ui.actionStart->setEnabled(true);
	ui.actionRecord->setEnabled(false);
}

void MainWindow::openFile() {
	QString fileName = QDir::toNativeSeparators(QFileDialog::getOpenFileName());
	if (!fileName.isEmpty()){
		fileOpened = true;
		trackController->openFile(fileName.toStdString());
		ui.actionCerrar_video->setEnabled(true);
	}
}

void MainWindow::startTracking() {
	if(fileOpened == true){
		Ret *r=Ret::getRet();
		r->setStop(false);
		r->setPause(false);
		r->setEndFile(false);
		ui.actionCerrar_video->setEnabled(false);
		settingsWidget->setPrevViewEnabled(false);
		trackController->startTracking();
		ui.actionStart->setEnabled(false);
		ui.actionRecord->setEnabled(true);
		ui.actionStop->setEnabled(true);
	}else{
		QMessageBox::StandardButton reply;
		reply = QMessageBox::critical(this, tr("Error"),
                                    tr("No se ha abierto ningn fichero de video."),
                                    QMessageBox::Ok);
		return;
	}
}

void MainWindow::stopTracking() {
	Ret *r=Ret::getRet();
	r->setStop(true);
	ui.actionCerrar_video->setEnabled(true);
	trackController->stopTracking();
	settingsWidget->setPrevViewEnabled(true);
	ui.actionStart->setEnabled(true);
	ui.actionRecord->setEnabled(false);
	ui.actionStop->setEnabled(false);
	renderWidget->reset();
	renderWidget->show();
}
	

void MainWindow::updateStats() {
	statusBar()->showMessage(QString("FPS: ")+QString::number(trackController->getFPS(), 'f', 1));
}
