/*	
	Proyecto TFG:
		AMH VIDA
	Ao:
		2011-2012.
	Version:
		1.0
	Fecha de ultima modificacion:
		18-Jul-2012.
	Fuente original:
		http://www.qtcentre.org/threads/11655-OpenCV-integration
	Modificacin:
		Jose Luis Martinez Jimenez.
	Tutores:
		Francisco Javier Finat Codes.
		Valentn Cardeoso Payo
	Descripcion:
		Capturethread recupera frames de un archivo de video
		a travs de la interfaz de OpenCV y los va publicando
		en un buffer (Imagebuffer).
*/

#ifndef CAPTURE_THREAD_H
#define CAPTURE_THREAD_H

#include <QThread>
#include <QMutex>
#include <QWaitCondition>
#include <QQueue>
#include "opencv/highgui.h"

#include <vector>
#include <string>

#include "ControladorNegocio.h"

class ImageBuffer;

class CaptureThread : public QThread {
Q_OBJECT;
public: 
	enum FrameSize { Size640, Size320 };
	CaptureThread(ImageBuffer* buffer);
	void run();
	bool startCapture(int framerate, FrameSize size);
	void stopCapture();
	void pauseCapture();
	double getFPS() { return fps; }
	bool isCapturing() { return captureActive; }
	void setFilename(std::string dir);
signals:
	void videoEnds();
private:
	void updateFPS(int time);
	QMutex captureLock;
	QWaitCondition captureWait;
	ImageBuffer* imageBuffer;
	bool captureActive;
	CvCapture* capture;
	double fps;
	QQueue<int> frameTimes;
	std::string filename;
	bool stop;
};

#endif
