/*	
	Proyecto TFG:
		AMH VIDA
	Ao:
		2011-2012.
	Version:
		1.0
	Fecha de ultima modificacion:
		18-Jul-2012.
	Fuente original:
		http://www.qtcentre.org/threads/11655-OpenCV-integration
	Modificacin:
		Jose Luis Martinez Jimenez.
	Tutores:
		Francisco Javier Finat Codes.
		Valentn Cardeoso Payo
	Descripcion:
		Renderwidget (Filter), encargado de dibujar por pantalla
		la imagen notificada, la transforma de IplImage (imagen 
		del formato OpenCV) a QImage (imagen del formato de QT),
		reescala y la pinta.
*/
#include "renderwidget.h"
#include <QPainter>
#include <QDebug>
#include <QTime>

RenderWidget::RenderWidget(QWidget* parent) : QWidget(parent), Filter(), imageData(0), imageWidth(0), imageHeight(0) {
	setAttribute(Qt::WA_OpaquePaintEvent, true); // don't clear the area before the paintEvent
	setAttribute(Qt::WA_PaintOnScreen, true); // disable double buffering
	//setFixedSize(imageWidth, imageHeight);
	connect(this, SIGNAL(frameSizeChanged(int, int)), this, SLOT(onFrameSizeChanged(int, int)), Qt::QueuedConnection);
	time = frames = 0;
}

void RenderWidget::onFrameSizeChanged(int width, int height) {
	setFixedSize(width, height);
} 

void RenderWidget::updatePixmap(const IplImage* frame) {
	QTime t;
	t.start();
	//qDebug() << "Copying data";
	bool start = false;
	// check if the frame dimensions have changed
	if(frame->width != imageWidth || frame->height != imageHeight) { 
		if(imageData) {
			delete[] imageData;
		}
		start = true;
		imageWidth = frame->width;
		imageHeight = frame->height;
		emit(frameSizeChanged(imageWidth, imageHeight));
		imageData = new unsigned char[4*imageWidth*imageHeight];
		for(int i = 0; i < imageWidth*imageHeight; i++) {
			imageData[i*4+3] = 0xFF;
		}
	}

	int pixels = imageWidth * imageHeight;
	uchar* src = (uchar*)(frame->imageData);
    uchar* srcEnd = src + (3*pixels);
    uchar* dest = imageData;

    do {
		memcpy(dest, src, 3);
		dest += 4;
		src += 3;
    } while(src < srcEnd);

	if(!start) {
		++frames;
		time += t.elapsed();
		//qDebug() << (double)time/frames;
	}
	//qDebug() << "Done Copying data";
}

void RenderWidget::processPoint(HeadState* state) {
	// copy the image to the local pixmap and update the display
	updatePixmap(state->frame);
	update();

	notifyListener(state);
}

void RenderWidget::paintEvent(QPaintEvent*) {
//	QTime time;
//	time.start(); 
	QPainter painter(this);

	Ret *r=Ret::getRet();
	if(imageData && r->getStop()==false) {
//QPixmap temp;
	//	qDebug() << temp.loadFromData(imageData, imageWidth*imageHeight*4);
		//qDebug() << "loading...";
		QImage tImg(imageData, imageWidth, imageHeight, QImage::Format_RGB32);
		emit(frameSizeChanged(1080, 720));
		tImg=tImg.scaled(1080,720, Qt::KeepAspectRatioByExpanding);
		painter.setBrush(Qt::black);
		painter.drawRect(rect());
		//emit(frameSizeChanged(rect().width(), rect().height()));
		//tImg=tImg.scaled(rect().width(), rect().height(), Qt::KeepAspectRatioByExpanding);

		int x = tImg.size().width();
		int y = tImg.size().height();
		//qDebug() << "drawing...";
		painter.drawImage(QPoint(0,0), tImg);
		//qDebug() << "done drawing";
	}
	else {
		painter.setBrush(Qt::black);
		painter.drawRect(rect());
	}
//	qDebug() << time.elapsed();
}

void RenderWidget::reset(){
	paintEvent(0);
}